@@ -47,14 +47,14 @@
 #define d(x)
 
 #define CAMEL_MAPI_FOLDER_LOCK(f, l) \
-	(g_static_mutex_lock(&((CamelMapiFolder *)f)->priv->l))
+	(g_mutex_lock(&((CamelMapiFolder *)f)->priv->l))
 #define CAMEL_MAPI_FOLDER_UNLOCK(f, l) \
-	(g_static_mutex_unlock(&((CamelMapiFolder *)f)->priv->l))
+	(g_mutex_unlock(&((CamelMapiFolder *)f)->priv->l))
 
 extern gint camel_application_is_exiting;
 
 struct _CamelMapiFolderPrivate {
-	GStaticMutex search_lock;	/* for locking the search object */
+	GMutex search_lock;	/* for locking the search object */
 
 	gchar *foreign_username;
 };
@@ -756,7 +756,7 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	CamelMapiStoreInfo *msi = NULL;
 	CamelMapiStore *mapi_store = CAMEL_MAPI_STORE (store);
 	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER (folder);
-	EMapiConnection *conn = camel_mapi_store_ref_connection (mapi_store, cancellable, mapi_error);
+	EMapiConnection *conn = camel_mapi_store_get_connection (mapi_store, cancellable, mapi_error);
 	struct FolderBasicPropertiesData fbp;
 	struct GatherChangedObjectsData gco;
 	mapi_object_t obj_folder;
@@ -782,7 +782,6 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 	if (!msi) {
 		camel_operation_pop_message (cancellable);
 		camel_folder_thaw (folder);
-		g_object_unref (conn);
 
 		g_return_val_if_fail (msi != NULL, FALSE);
 
@@ -883,7 +882,6 @@ camel_mapi_folder_fetch_summary (CamelFolder *folder, GCancellable *cancellable,
 		msi->last_obj_total = fbp.obj_total;
 	}
 
-	g_object_unref (conn);
 	if (mapi_error && *mapi_error)
 		camel_mapi_store_maybe_disconnect (mapi_store, *mapi_error);
 
@@ -1107,6 +1105,17 @@ mapi_folder_dispose (GObject *object)
 	G_OBJECT_CLASS (camel_mapi_folder_parent_class)->dispose (object);
 }
 
+static void
+mapi_folder_finalize (GObject *object)
+{
+	CamelMapiFolder *mapi_folder = CAMEL_MAPI_FOLDER (object);
+
+	g_mutex_clear (&mapi_folder->priv->search_lock);
+
+	/* Chain up to parent's finalize() method. */
+	G_OBJECT_CLASS (camel_mapi_folder_parent_class)->finalize (object);
+}
+
 static void
 mapi_folder_constructed (GObject *object)
 {
@@ -1208,7 +1217,7 @@ mapi_folder_append_message_sync (CamelFolder *folder,
 		return FALSE;
 	}
 
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
 	if (!conn) {
 		g_set_error (
 			error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
@@ -1234,8 +1243,6 @@ mapi_folder_append_message_sync (CamelFolder *folder,
 	if (mid) {
 		mapi_refresh_folder (folder, cancellable, error);
 	} else {
-		g_object_unref (conn);
-
 		if (mapi_error) {
 			if (!e_mapi_utils_propagate_cancelled_error (mapi_error, error))
 				g_set_error_literal (error, CAMEL_ERROR, CAMEL_ERROR_GENERIC, mapi_error->message);
@@ -1248,8 +1255,6 @@ mapi_folder_append_message_sync (CamelFolder *folder,
 		return FALSE;
 	}
 
-	g_object_unref (conn);
-
 	if (appended_uid)
 		*appended_uid = e_mapi_util_mapi_id_to_string (mid);
 
@@ -1281,7 +1286,7 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 
 	mapi_folder = CAMEL_MAPI_FOLDER (folder);
 	mapi_store = CAMEL_MAPI_STORE (parent_store);
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
 
 	if (!conn)
 		return FALSE;
@@ -1334,8 +1339,6 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 				_("Failed to empty Trash"));
 		}
 
-		g_object_unref (conn);
-
 		return status;
 	}
 
@@ -1406,7 +1409,6 @@ mapi_folder_expunge_sync (CamelFolder *folder,
 		camel_folder_changed (folder, changes);
 
 	camel_folder_change_info_free (changes);
-	g_object_unref (conn);
 
 	return TRUE;
 }
@@ -1541,7 +1543,7 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 		return NULL;
 	}
 
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
 	if (!conn)
 		return NULL;
 
@@ -1554,8 +1556,6 @@ mapi_folder_get_message_sync (CamelFolder *folder,
 		e_mapi_connection_close_folder (conn, &obj_folder, cancellable, NULL);
 	}
 
-	g_object_unref (conn);
-
 	if (!msg) {
 		if (mapi_error) {
 			if (!e_mapi_utils_propagate_cancelled_error (mapi_error, error))
@@ -1632,7 +1632,7 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	folder_id =  camel_mapi_store_folder_id_lookup (mapi_store, full_name);
 	e_mapi_util_mapi_id_from_string (folder_id, &fid);
 
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
 	if (!conn)
 		return FALSE;
 
@@ -1786,8 +1786,6 @@ mapi_folder_synchronize_sync (CamelFolder *folder,
 	g_slist_foreach (to_free, (GFunc) g_free, NULL);
 	g_slist_free (to_free);
 
-	g_object_unref (conn);
-
 	if (mapi_error) {
 		camel_mapi_store_maybe_disconnect (mapi_store, mapi_error);
 		g_clear_error (&mapi_error);
@@ -1831,16 +1829,13 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 
 	source_parent_store = camel_folder_get_parent_store (source);
 	mapi_store = CAMEL_MAPI_STORE (source_parent_store);
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
 
 	if (!conn || !CAMEL_IS_MAPI_FOLDER (source) || !CAMEL_IS_MAPI_FOLDER (destination) ||
 	    (CAMEL_MAPI_FOLDER (source)->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0 ||
 	    (CAMEL_MAPI_FOLDER (destination)->mapi_folder_flags & CAMEL_MAPI_STORE_FOLDER_FLAG_PUBLIC) != 0) {
 		CamelFolderClass *folder_class;
 
-		if (conn)
-			g_object_unref (conn);
-
 		/* because cannot use MAPI to copy/move messages with public folders,
 		   thus fallback to per-message copy/move */
 		folder_class = CAMEL_FOLDER_CLASS (camel_mapi_folder_parent_class);
@@ -1857,10 +1852,8 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	offline = CAMEL_OFFLINE_STORE (destination_parent_store);
 
 	/* check for offline operation */
-	if (!camel_offline_store_get_online (offline)) {
-		g_object_unref (conn);
+	if (!camel_offline_store_get_online (offline))
 		return FALSE;
-	}
 
 	src_mapi_folder = CAMEL_MAPI_FOLDER (source);
 	des_mapi_folder = CAMEL_MAPI_FOLDER (destination);
@@ -1907,8 +1900,6 @@ mapi_folder_transfer_messages_to_sync (CamelFolder *source,
 	g_slist_foreach (src_msg_ids, (GFunc) g_free, NULL);
 	g_slist_free (src_msg_ids);
 
-	g_object_unref (conn);
-
 	/* update the destination folder to notice new messages */
 	if (success)
 		success = mapi_folder_refresh_info_sync (destination, cancellable, error);
@@ -1937,8 +1928,11 @@ mapi_folder_get_quota_info_sync (CamelFolder *folder,
 	if (!camel_offline_store_get_online (CAMEL_OFFLINE_STORE (mapi_store)))
 		return NULL;
 
-	conn = camel_mapi_store_ref_connection (mapi_store, cancellable, error);
-	if (conn && e_mapi_connection_get_store_quotas (conn, NULL, &current_size, &receive_quota, &send_quota, cancellable, &mapi_error)) {
+	conn = camel_mapi_store_get_connection (mapi_store, cancellable, error);
+	if (conn && e_mapi_connection_get_store_quotas (
+		conn, NULL,
+		&current_size, &receive_quota, &send_quota,
+		cancellable, &mapi_error)) {
 		if (current_size != -1) {
 			if (receive_quota != -1) {
 				quota_info = camel_folder_quota_info_new (_("Receive quota"), current_size, receive_quota);
@@ -1956,9 +1950,6 @@ mapi_folder_get_quota_info_sync (CamelFolder *folder,
 		}
 	}
 
-	if (conn)
-		g_object_unref (conn);
-
 	if (!quota_info) {
 		if (mapi_error) {
 			if (!e_mapi_utils_propagate_cancelled_error (mapi_error, error))
@@ -1986,6 +1977,7 @@ camel_mapi_folder_class_init (CamelMapiFolderClass *class)
 
 	object_class = G_OBJECT_CLASS (class);
 	object_class->dispose = mapi_folder_dispose;
+	object_class->finalize = mapi_folder_finalize;
 	object_class->constructed = mapi_folder_constructed;
 
 	folder_class = CAMEL_FOLDER_CLASS (class);
@@ -2017,9 +2009,7 @@ camel_mapi_folder_init (CamelMapiFolder *mapi_folder)
 
 	folder->folder_flags = CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY;
 
-#ifdef ENABLE_THREADS
-	g_static_mutex_init(&mapi_folder->priv->search_lock);
-#endif
+	g_mutex_init (&mapi_folder->priv->search_lock);
 
 	mapi_folder->need_rescan = TRUE;
 }
